@@ -415,6 +415,30 @@ AUTOMATE(leaves, "", options::opts::none)
     output << (*i).inner()() << '\n';
 }
 
+// Name: roots
+// Arguments:
+//   None
+// Added in: 4.3
+// Purpose: Prints the roots of the revision graph, i.e. all revisions that
+//   have no parents.
+// Output format: A list of revision ids, in hexadecimal, each followed by a
+//   newline.  Revision ids are printed in alphabetically sorted order.
+// Error conditions: None.
+AUTOMATE(roots, "", options::opts::none)
+{
+  N(args.size() == 0,
+    F("no arguments needed"));
+
+  // the real root revisions are the children of one single imaginary root
+  // with an empty revision id
+  set<revision_id> roots;
+  revision_id nullid;
+  app.db.get_revision_children(nullid, roots);
+  for (set<revision_id>::const_iterator i = roots.begin();
+       i != roots.end(); ++i)
+      output << i->inner()() << '\n';
+}
+
 // Name: parents
 // Arguments:
 //   1: a revision id
@@ -1630,8 +1654,7 @@ AUTOMATE(put_file, N_("[FILEID] CONTENTS"), options::opts::none)
       file_data dat(idx(args, 0)());
       calculate_ident(dat, sha1sum);
       
-      if (!app.db.file_version_exists(sha1sum))
-        app.db.put_file(sha1sum, dat);
+      app.db.put_file(sha1sum, dat);
     }
   else if (args.size() == 2)
     {
@@ -1640,19 +1663,17 @@ AUTOMATE(put_file, N_("[FILEID] CONTENTS"), options::opts::none)
       file_id base_id(idx(args, 0)());
       N(app.db.file_version_exists(base_id),
         F("no file version %s found in database") % base_id);
-     
+
+      // put_file_version won't do anything if the target ID already exists,
+      // but we can save the delta calculation by checking here too
       if (!app.db.file_version_exists(sha1sum))
         {
           file_data olddat;
           app.db.get_file_version(base_id, olddat);
           delta del;
           diff(olddat.inner(), dat.inner(), del);
-          L(FL("data size %d, delta size %d") % dat.inner()().size() % del().size());
-          if (dat.inner()().size() <= del().size())
-            // the data is smaller or of equal size to the patch
-            app.db.put_file(sha1sum, dat);
-          else
-            app.db.put_file_version(base_id, sha1sum, file_delta(del));
+
+          app.db.put_file_version(base_id, sha1sum, file_delta(del));
         }
     }
   else I(false);
@@ -1705,15 +1726,10 @@ AUTOMATE(put_revision, N_("REVISION-DATA"), options::opts::none)
   revision_id id;
   calculate_ident(rev, id);
 
-  if (app.db.revision_exists(id))
-    P(F("revision %s already present in the database, skipping") % id);
-  else
-    {
-      transaction_guard tr(app.db);
-      rev.made_for = made_for_database;
-      app.db.put_revision(id, rev);
-      tr.commit();
-    }
+  // If the database refuses the revision, make sure this is because it's
+  // already there.
+  E(app.db.put_revision(id, rev) || app.db.revision_exists(id),
+    F("missing prerequisite for revision %s") % id);
 
   output << id << '\n';
 }
@@ -1744,8 +1760,7 @@ AUTOMATE(cert, N_("REVISION-ID NAME VALUE"), options::opts::none)
   make_simple_cert(rid.inner(), cert_name(idx(args, 1)()),
                    cert_value(idx(args, 2)()), app, c);
   revision<cert> rc(c);
-  packet_db_writer dbw(app);
-  dbw.consume_revision_cert(rc);
+  app.db.put_revision_cert(rc);
   guard.commit();
 }
 
