@@ -1,12 +1,13 @@
 /*************************************************
 * BigInt Binary Operators Source File            *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #include <botan/bigint.h>
 #include <botan/numthry.h>
 #include <botan/mp_core.h>
 #include <botan/bit_ops.h>
+#include <algorithm>
 
 namespace Botan {
 
@@ -15,17 +16,28 @@ namespace Botan {
 *************************************************/
 BigInt operator+(const BigInt& x, const BigInt& y)
    {
+   const u32bit x_sw = x.sig_words(), y_sw = y.sig_words();
+
+   BigInt z(x.sign(), std::max(x_sw, y_sw) + 1);
+
    if((x.sign() == y.sign()))
-      {
-      BigInt z(x.sign(), std::max(x.sig_words(), y.sig_words()) + 1);
-      bigint_add3(z.get_reg(), x.data(), x.sig_words(),
-                               y.data(), y.sig_words());
-      return z;
-      }
-   else if(x.is_positive())
-      return (x - y.abs());
+      bigint_add3(z.get_reg(), x.data(), x_sw, y.data(), y_sw);
    else
-      return (y - x.abs());
+      {
+      s32bit relative_size = bigint_cmp(x.data(), x_sw, y.data(), y_sw);
+
+      if(relative_size < 0)
+         {
+         bigint_sub3(z.get_reg(), y.data(), y_sw, x.data(), x_sw);
+         z.set_sign(y.sign());
+         }
+      else if(relative_size == 0)
+         z.set_sign(BigInt::Positive);
+      else if(relative_size > 0)
+         bigint_sub3(z.get_reg(), x.data(), x_sw, y.data(), y_sw);
+      }
+
+   return z;
    }
 
 /*************************************************
@@ -33,31 +45,31 @@ BigInt operator+(const BigInt& x, const BigInt& y)
 *************************************************/
 BigInt operator-(const BigInt& x, const BigInt& y)
    {
-   s32bit relative_size = bigint_cmp(x.data(), x.sig_words(),
-                                     y.data(), y.sig_words());
-   if(relative_size == 0 && (x.sign() == y.sign())) return 0;
-   if(relative_size == 0 && (x.sign() != y.sign())) return (x << 1);
+   const u32bit x_sw = x.sig_words(), y_sw = y.sig_words();
 
-   BigInt z(BigInt::Positive, std::max(x.sig_words(), y.sig_words()) + 1);
+   s32bit relative_size = bigint_cmp(x.data(), x_sw, y.data(), y_sw);
 
-   if(relative_size == -1)
+   BigInt z(BigInt::Positive, std::max(x_sw, y_sw) + 1);
+
+   if(relative_size < 0)
       {
       if(x.sign() == y.sign())
-         bigint_sub3(z.get_reg(), y.data(), y.sig_words(),
-                                  x.data(), x.sig_words());
+         bigint_sub3(z.get_reg(), y.data(), y_sw, x.data(), x_sw);
       else
-         bigint_add3(z.get_reg(), x.data(), x.sig_words(),
-                                  y.data(), y.sig_words());
+         bigint_add3(z.get_reg(), x.data(), x_sw, y.data(), y_sw);
       z.set_sign(y.reverse_sign());
       }
-   if(relative_size == 1)
+   else if(relative_size == 0)
+      {
+      if(x.sign() != y.sign())
+         bigint_shl2(z.get_reg(), x.data(), x_sw, 0, 1);
+      }
+   else if(relative_size > 0)
       {
       if(x.sign() == y.sign())
-         bigint_sub3(z.get_reg(), x.data(), x.sig_words(),
-                                  y.data(), y.sig_words());
+         bigint_sub3(z.get_reg(), x.data(), x_sw, y.data(), y_sw);
       else
-         bigint_add3(z.get_reg(), x.data(), x.sig_words(),
-                                  y.data(), y.sig_words());
+         bigint_add3(z.get_reg(), x.data(), x_sw, y.data(), y_sw);
       z.set_sign(x.sign());
       }
    return z;
@@ -68,30 +80,24 @@ BigInt operator-(const BigInt& x, const BigInt& y)
 *************************************************/
 BigInt operator*(const BigInt& x, const BigInt& y)
    {
-   if(x.is_zero() || y.is_zero())
-      return 0;
+   const u32bit x_sw = x.sig_words(), y_sw = y.sig_words();
 
-   BigInt::Sign sign = BigInt::Positive;
-   if(x.sign() != y.sign())
-      sign = BigInt::Negative;
+   BigInt z(BigInt::Positive, x.size() + y.size());
 
-   const u32bit x_sw = x.sig_words();
-   const u32bit y_sw = y.sig_words();
-
-   if(x_sw == 1 || y_sw == 1)
+   if(x_sw == 1 && y_sw)
+      bigint_linmul3(z.get_reg(), y.data(), y_sw, x.word_at(0));
+   else if(y_sw == 1 && x_sw)
+      bigint_linmul3(z.get_reg(), x.data(), x_sw, y.word_at(0));
+   else if(x_sw && y_sw)
       {
-      BigInt z(sign, x_sw + y_sw);
-      if(x_sw == 1)
-         bigint_linmul3(z.get_reg(), y.data(), y_sw, x.word_at(0));
-      else
-         bigint_linmul3(z.get_reg(), x.data(), x_sw, y.word_at(0));
-      return z;
+      SecureVector<word> workspace(z.size());
+      bigint_mul(z.get_reg(), z.size(), workspace,
+                 x.data(), x.size(), x_sw,
+                 y.data(), y.size(), y_sw);
       }
 
-   BigInt z(sign, x.size() + y.size());
-   bigint_mul3(z.get_reg(), z.size(),
-               x.data(), x.size(), x_sw,
-               y.data(), y.size(), y_sw);
+   if(x_sw && y_sw && x.sign() != y.sign())
+      z.flip_sign();
    return z;
    }
 
@@ -129,15 +135,16 @@ word operator%(const BigInt& n, word mod)
    {
    if(mod == 0)
       throw BigInt::DivideByZero();
-
    if(power_of_2(mod))
       return (n.word_at(0) & (mod - 1));
 
    word remainder = 0;
-   u32bit size = n.sig_words();
 
-   for(u32bit j = size; j > 0; j--)
+   for(u32bit j = n.sig_words(); j > 0; --j)
       remainder = bigint_modop(remainder, n.word_at(j-1), mod);
+
+   if(remainder && n.sign() == BigInt::Negative)
+      return mod - remainder;
    return remainder;
    }
 
@@ -146,12 +153,16 @@ word operator%(const BigInt& n, word mod)
 *************************************************/
 BigInt operator<<(const BigInt& x, u32bit shift)
    {
-   if(shift == 0) return x;
+   if(shift == 0)
+      return x;
+
    const u32bit shift_words = shift / MP_WORD_BITS,
                 shift_bits  = shift % MP_WORD_BITS;
 
-   BigInt y(x.sign(), x.sig_words() + shift_words + (shift_bits ? 1 : 0));
-   bigint_shl2(y.get_reg(), x.data(), x.sig_words(), shift_words, shift_bits);
+   const u32bit x_sw = x.sig_words();
+
+   BigInt y(x.sign(), x_sw + shift_words + (shift_bits ? 1 : 0));
+   bigint_shl2(y.get_reg(), x.data(), x_sw, shift_words, shift_bits);
    return y;
    }
 
@@ -160,13 +171,17 @@ BigInt operator<<(const BigInt& x, u32bit shift)
 *************************************************/
 BigInt operator>>(const BigInt& x, u32bit shift)
    {
-   if(shift == 0)        return x;
-   if(x.bits() <= shift) return 0;
+   if(shift == 0)
+      return x;
+   if(x.bits() <= shift)
+      return 0;
 
    const u32bit shift_words = shift / MP_WORD_BITS,
-                shift_bits  = shift % MP_WORD_BITS;
-   BigInt y(x.sign(), x.sig_words() - shift_words);
-   bigint_shr2(y.get_reg(), x.data(), x.sig_words(), shift_words, shift_bits);
+                shift_bits  = shift % MP_WORD_BITS,
+                x_sw = x.sig_words();
+
+   BigInt y(x.sign(), x_sw - shift_words);
+   bigint_shr2(y.get_reg(), x.data(), x_sw, shift_words, shift_bits);
    return y;
    }
 
