@@ -1,14 +1,15 @@
 /*************************************************
 * Prime Generation Source File                   *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #include <botan/numthry.h>
+#include <botan/libstate.h>
 #include <botan/lookup.h>
 #include <botan/bit_ops.h>
 #include <botan/parsing.h>
 #include <botan/rng.h>
-#include <botan/ui.h>
+#include <algorithm>
 #include <memory>
 
 namespace Botan {
@@ -20,7 +21,7 @@ namespace {
 *************************************************/
 void increment(SecureVector<byte>& seed)
    {
-   for(u32bit j = seed.size(); j > 0; j--)
+   for(u32bit j = seed.size(); j > 0; --j)
       if(++seed[j-1])
          break;
    }
@@ -54,21 +55,21 @@ bool generate_dsa_primes(BigInt& p, BigInt& q, const byte const_seed[],
    q.binary_decode(qhash, qhash.size());
    if(!is_prime(q))
       return false;
-   UI::pulse(UI::PRIME_FOUND);
+   global_state().pulse(PRIME_FOUND);
 
    u32bit n = (pbits-1) / 160, b = (pbits-1) % 160;
    SecureVector<byte> W(20 * (n+1));
    BigInt X;
 
-   for(u32bit j = 0; j != counter_start; j++)
-      for(u32bit k = 0; k != n + 1; k++)
+   for(u32bit j = 0; j != counter_start; ++j)
+      for(u32bit k = 0; k != n + 1; ++k)
          increment(seed);
 
-   for(u32bit j = 0; j != 4096 - counter_start; j++)
+   for(u32bit j = 0; j != 4096 - counter_start; ++j)
       {
-      UI::pulse(UI::PRIME_SEARCHING);
+      global_state().pulse(PRIME_SEARCHING);
 
-      for(u32bit k = 0; k != n + 1; k++)
+      for(u32bit k = 0; k != n + 1; ++k)
          {
          increment(seed);
          sha1->update(seed);
@@ -81,7 +82,7 @@ bool generate_dsa_primes(BigInt& p, BigInt& q, const byte const_seed[],
 
       if(p.bits() == pbits && is_prime(p))
          {
-         UI::pulse(UI::PRIME_FOUND);
+         global_state().pulse(PRIME_FOUND);
          return true;
          }
       }
@@ -97,8 +98,8 @@ SecureVector<byte> generate_dsa_primes(BigInt& p, BigInt& q, u32bit pbits)
 
    while(true)
       {
-      Global_RNG::randomize(seed, seed.size(), Nonce);
-      UI::pulse(UI::PRIME_SEARCHING);
+      Global_RNG::randomize(seed, seed.size());
+      global_state().pulse(PRIME_SEARCHING);
       if(generate_dsa_primes(p, q, seed, seed.size(), pbits))
          return seed;
       }
@@ -107,10 +108,10 @@ SecureVector<byte> generate_dsa_primes(BigInt& p, BigInt& q, u32bit pbits)
 /*************************************************
 * Generate a random prime                        *
 *************************************************/
-BigInt random_prime(u32bit bits, RNG_Quality level, const BigInt& coprime,
+BigInt random_prime(u32bit bits, const BigInt& coprime,
                     u32bit equiv, u32bit modulo)
    {
-   if(bits <= 48)
+   if(bits < 48)
       throw Invalid_Argument("random_prime: Can't make a prime of " +
                              to_string(bits) + " bits");
 
@@ -123,9 +124,9 @@ BigInt random_prime(u32bit bits, RNG_Quality level, const BigInt& coprime,
 
    while(true)
       {
-      UI::pulse(UI::PRIME_SEARCHING);
+      global_state().pulse(PRIME_SEARCHING);
 
-      BigInt p = random_integer(bits, level);
+      BigInt p = random_integer(bits);
       p.set_bit(bits - 2);
       p.set_bit(0);
 
@@ -135,10 +136,10 @@ BigInt random_prime(u32bit bits, RNG_Quality level, const BigInt& coprime,
       const u32bit sieve_size = std::min(bits / 2, PRIME_TABLE_SIZE);
       SecureVector<u32bit> sieve(sieve_size);
 
-      for(u32bit j = 0; j != sieve.size(); j++)
+      for(u32bit j = 0; j != sieve.size(); ++j)
          {
          sieve[j] = p % PRIMES[j];
-         UI::pulse(UI::PRIME_SIEVING);
+         global_state().pulse(PRIME_SIEVING);
          }
 
       u32bit counter = 0;
@@ -147,26 +148,26 @@ BigInt random_prime(u32bit bits, RNG_Quality level, const BigInt& coprime,
          if(counter == 4096 || p.bits() > bits)
             break;
 
-         UI::pulse(UI::PRIME_SEARCHING);
+         global_state().pulse(PRIME_SEARCHING);
 
          bool passes_sieve = true;
-         counter++;
+         ++counter;
          p += modulo;
 
-         for(u32bit j = 0; j != sieve.size(); j++)
+         for(u32bit j = 0; j != sieve.size(); ++j)
             {
             sieve[j] = (sieve[j] + modulo) % PRIMES[j];
-            UI::pulse(UI::PRIME_SIEVING);
+            global_state().pulse(PRIME_SIEVING);
             if(sieve[j] == 0)
                passes_sieve = false;
             }
 
          if(!passes_sieve || gcd(p - 1, coprime) != 1)
             continue;
-         UI::pulse(UI::PRIME_PASSED_SIEVE);
+         global_state().pulse(PRIME_PASSED_SIEVE);
          if(passes_mr_tests(p))
             {
-            UI::pulse(UI::PRIME_FOUND);
+            global_state().pulse(PRIME_FOUND);
             return p;
             }
          }
