@@ -1,12 +1,12 @@
 /*************************************************
 * Low Level Types Header File                    *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2006 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_TYPES_H__
 #define BOTAN_TYPES_H__
 
-#include <botan/config.h>
+#include <botan/build.h>
 
 namespace Botan {
 
