@@ -10,12 +10,13 @@
 #include <iostream>
 
 #include "annotate.hh"
+#include "revision.hh"
 #include "cmd.hh"
 #include "diff_patch.hh"
 #include "file_io.hh"
-#include "packet.hh"
 #include "simplestring_xform.hh"
 #include "transforms.hh"
+#include "app_state.hh"
 
 using std::cout;
 using std::ostream_iterator;
@@ -35,8 +36,9 @@ CMD(fload, N_("debug"), "", N_("load file contents into db"), options::opts::non
 
   calculate_ident (f_data, f_id);
 
-  packet_db_writer dbw(app);
-  dbw.consume_file_data(f_id, f_data);
+  transaction_guard guard(app.db);
+  app.db.put_file(f_id, f_data);
+  guard.commit();
 }
 
 CMD(fmerge, N_("debug"), N_("<parent> <left> <right>"),
@@ -203,6 +205,35 @@ CMD(identify, N_("debug"), N_("[PATH]"),
   cout << ident << '\n';
 }
 
+// Name: identify
+// Arguments:
+//   1: a file path
+// Added in: 4.2
+// Purpose: Prints the fileid of the given file (aka hash)
+//
+// Output format: a single, 40 byte long hex-encoded id
+//
+// Error conditions: If the file path doesn't point to a valid file prints
+// an error message to stderr and exits with status 1.
+AUTOMATE(identify, N_("PATH"), options::opts::none)
+{
+  N(args.size() == 1,
+    F("wrong argument count"));
+  
+  utf8 path = idx(args, 0);
+  
+  N(path() != "-",
+    F("Cannot read from stdin"));
+  
+  data dat;
+  read_data_for_command_line(path, dat);
+  
+  hexenc<id> ident;
+  calculate_ident(dat, ident);
+  
+  output << ident << '\n';
+}
+
 static void
 dump_file(std::ostream & output, app_state & app, file_id & ident)
 {
