@@ -13,12 +13,13 @@
 #include "cmd.hh"
 #include "diff_patch.hh"
 #include "file_io.hh"
-#include "packet.hh"
 #include "restrictions.hh"
 #include "revision.hh"
 #include "transforms.hh"
 #include "work.hh"
 #include "charset.hh"
+#include "ui.hh"
+#include "app_state.hh"
 
 using std::cout;
 using std::make_pair;
@@ -31,15 +32,78 @@ using std::vector;
 
 using boost::shared_ptr;
 
+static void
+revision_summary(revision_t const & rev, branch_name const & branch, utf8 & summary)
+{
+  string out;
+  // We intentionally do not collapse the final \n into the format
+  // strings here, for consistency with newline conventions used by most
+  // other format strings.
+  out += (F("Current branch: %s") % branch).str() += '\n';
+  for (edge_map::const_iterator i = rev.edges.begin(); i != rev.edges.end(); ++i)
+    {
+      revision_id parent = edge_old_revision(*i);
+      // A colon at the end of this string looked nicer, but it made
+      // double-click copying from terminals annoying.
+      out += (F("Changes against parent %s") % parent).str() += '\n';
+
+      cset const & cs = edge_changes(*i);
+
+      if (cs.empty())
+        out += F("  no changes").str() += '\n';
+
+      for (path_set::const_iterator i = cs.nodes_deleted.begin();
+            i != cs.nodes_deleted.end(); ++i)
+        out += (F("  dropped  %s") % *i).str() += '\n';
+
+      for (map<split_path, split_path>::const_iterator
+            i = cs.nodes_renamed.begin();
+            i != cs.nodes_renamed.end(); ++i)
+        out += (F("  renamed  %s\n"
+                   "       to  %s") % i->first % i->second).str() += '\n';
+
+      for (path_set::const_iterator i = cs.dirs_added.begin();
+            i != cs.dirs_added.end(); ++i)
+        out += (F("  added    %s") % *i).str() += '\n';
+
+      for (map<split_path, file_id>::const_iterator i = cs.files_added.begin();
+            i != cs.files_added.end(); ++i)
+        out += (F("  added    %s") % i->first).str() += '\n';
+
+      for (map<split_path, pair<file_id, file_id> >::const_iterator
+              i = cs.deltas_applied.begin(); i != cs.deltas_applied.end(); ++i)
+        out += (F("  patched  %s") % (i->first)).str() += '\n';
+
+      for (map<pair<split_path, attr_key>, attr_value >::const_iterator
+             i = cs.attrs_set.begin(); i != cs.attrs_set.end(); ++i)
+        out += (F("  attr on  %s\n"
+                   "    attr   %s\n"
+                   "    value  %s")
+                 % (i->first.first) % (i->first.second) % (i->second)
+                 ).str() += "\n";
+
+      for (set<pair<split_path, attr_key> >::const_iterator
+             i = cs.attrs_cleared.begin(); i != cs.attrs_cleared.end(); ++i)
+        out += (F("  unset on %s\n"
+                   "      attr %s")
+                 % (i->first) % (i->second)).str() += "\n";
+    }
+    summary = utf8(out);
+}
+
 static void
 get_log_message_interactively(revision_t const & cs,
                               app_state & app,
                               utf8 & log_message)
 {
-  revision_data summary;
-  write_revision(cs, summary);
+  utf8 summary;
+  revision_summary(cs, app.opts.branchname, summary);
   external summary_external;
-  utf8_to_system_best_effort(utf8(summary.inner()()), summary_external);
+  utf8_to_system_best_effort(summary, summary_external);
+  
+  utf8 branch_comment = utf8((F("branch \"%s\"\n\n") % app.opts.branchname).str());
+  external branch_external;
+  utf8_to_system_best_effort(branch_comment, branch_external);
 
   string magic_line = _("*****DELETE THIS LINE TO CONFIRM YOUR COMMIT*****");
   string commentary_str;
@@ -245,19 +309,17 @@ CMD(disapprove, N_("review"), N_("REVISION"),
 
   {
     transaction_guard guard(app.db);
-    packet_db_writer dbw(app);
 
     revision_id inv_id;
     revision_data rdat;
 
     write_revision(rev_inverse, rdat);
     calculate_ident(rdat, inv_id);
-    dbw.consume_revision_data(inv_id, rdat);
+    app.db.put_revision(inv_id, rdat);
 
     app.get_project().put_standard_certs_from_options(inv_id,
                                                       app.opts.branchname,
-                                                      log_message,
-                                                      dbw);
+                                                      log_message);
     guard.commit();
   }
 }
@@ -303,7 +365,7 @@ CMD(mkdir, N_("workspace"), N_("[DIRECTORY...]"),
       mkdir_p(file_path(*i));
     }
 
-  app.work.perform_additions(paths, false, true);
+  app.work.perform_additions(paths, false, !app.opts.no_ignore);
 }
 
 CMD(add, N_("workspace"), N_("[PATH]..."),
@@ -433,56 +495,11 @@ CMD(status, N_("informative"), N_("[PATH]..."), N_("show status of workspace"),
   app.work.update_current_roster_from_filesystem(new_roster, mask);
   make_restricted_revision(old_rosters, new_roster, mask, rev);
 
-  // We intentionally do not collapse the final \n into the format
-  // strings here, for consistency with newline conventions used by most
-  // other format strings.
-  cout << (F("Current branch: %s") % app.opts.branchname).str() << '\n';
-  for (edge_map::const_iterator i = rev.edges.begin(); i != rev.edges.end(); ++i)
-    {
-      revision_id parent = edge_old_revision(*i);
-      // A colon at the end of this string looked nicer, but it made
-      // double-click copying from terminals annoying.
-      cout << (F("Changes against parent %s") % parent).str() << '\n';
-
-      cset const & cs = edge_changes(*i);
-
-      if (cs.empty())
-        cout << F("  no changes").str() << '\n';
-
-      for (path_set::const_iterator i = cs.nodes_deleted.begin();
-            i != cs.nodes_deleted.end(); ++i)
-        cout << (F("  dropped  %s") % *i).str() << '\n';
-
-      for (map<split_path, split_path>::const_iterator
-            i = cs.nodes_renamed.begin();
-            i != cs.nodes_renamed.end(); ++i)
-        cout << (F("  renamed  %s\n"
-                   "       to  %s") % i->first % i->second).str() << '\n';
-
-      for (path_set::const_iterator i = cs.dirs_added.begin();
-            i != cs.dirs_added.end(); ++i)
-        cout << (F("  added    %s") % *i).str() << '\n';
-
-      for (map<split_path, file_id>::const_iterator i = cs.files_added.begin();
-            i != cs.files_added.end(); ++i)
-        cout << (F("  added    %s") % i->first).str() << '\n';
-
-      for (map<split_path, pair<file_id, file_id> >::const_iterator
-              i = cs.deltas_applied.begin(); i != cs.deltas_applied.end(); ++i)
-        cout << (F("  patched  %s") % (i->first)).str() << '\n';
-
-      for (map<pair<split_path, attr_key>, attr_value >::const_iterator
-             i = cs.attrs_set.begin(); i != cs.attrs_set.end(); ++i)
-        cout << (F("  set on   %s\n"
-                   "    attr   %s")
-                 % (i->first.first) % (i->first.second)).str() << "\n";
-
-      for (set<pair<split_path, attr_key> >::const_iterator
-             i = cs.attrs_cleared.begin(); i != cs.attrs_cleared.end(); ++i)
-        cout << (F("  unset on %s\n"
-                   "      attr %s")
-                 % (i->first) % (i->second)).str() << "\n";
-    }
+  utf8 summary;
+  revision_summary(rev, app.opts.branchname, summary);
+  external summary_external;
+  utf8_to_system_best_effort(summary, summary_external);
+  cout << summary_external;
 }
 
 CMD(checkout, N_("tree"), N_("[DIRECTORY]"),
@@ -812,7 +829,6 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
   
   {
     transaction_guard guard(app.db);
-    packet_db_writer dbw(app);
 
     if (app.db.revision_exists(restricted_rev_id))
       W(F("revision %s already in database") % restricted_rev_id);
@@ -856,13 +872,12 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
                       % path);
                     delta del;
                     diff(old_data.inner(), new_data, del);
-                    dbw.consume_file_delta(old_content,
-                                           new_content,
-                                           file_delta(del));
+                    app.db.put_file_version(old_content,
+                                            new_content,
+                                            file_delta(del));
                   }
                 else
-                  // If we don't err out here, our packet writer will
-                  // later.
+                  // If we don't err out here, the database will later.
                   E(false,
                     F("Your database is missing version %s of file '%s'")
                     % old_content % path);
@@ -884,19 +899,18 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
                 N(tid == new_content.inner(),
                   F("file '%s' modified during commit, aborting")
                   % path);
-                dbw.consume_file_data(new_content, file_data(new_data));
+                app.db.put_file(new_content, file_data(new_data));
               }
           }
 
         revision_data rdat;
         write_revision(restricted_rev, rdat);
-        dbw.consume_revision_data(restricted_rev_id, rdat);
+        app.db.put_revision(restricted_rev_id, rdat);
       }
 
     app.get_project().put_standard_certs_from_options(restricted_rev_id,
                                                       app.opts.branchname,
-                                                      log_message,
-                                                      dbw);
+                                                      log_message);
     guard.commit();
   }
 
