@@ -80,7 +80,7 @@ get_passphrase(lua_hooks & lua,
                utf8 & phrase,
                bool confirm_phrase,
                bool force_from_user,
-               string prompt_beginning)
+               bool generating_key)
 {
 
   // we permit the user to relax security here, by caching a passphrase (if
@@ -113,8 +113,12 @@ get_passphrase(lua_hooks & lua,
           memset(pass1, 0, constants::maxpasswd);
           memset(pass2, 0, constants::maxpasswd);
           ui.ensure_clean_line();
-          read_password(prompt_beginning + " for key ID [" + keyid() + "]: ",
-                        pass1, constants::maxpasswd);
+          string prompt1 = ((confirm_phrase && !generating_key
+                             ? F("enter new passphrase for key ID [%s]: ")
+                             : F("enter passphrase for key ID [%s]: "))
+                            % keyid()).str();
+
+          read_password(prompt1, pass1, constants::maxpasswd);
           if (confirm_phrase)
             {
               ui.ensure_clean_line();
@@ -163,7 +167,7 @@ generate_key_pair(lua_hooks & lua,              // to hook for phrase
                   keypair & kp_out)
 {
   utf8 phrase;
-  get_passphrase(lua, id, phrase, true, true);
+  get_passphrase(lua, id, phrase, true, true, true);
   generate_key_pair(kp_out, phrase);
 }
 
@@ -253,7 +257,7 @@ get_private_key(lua_hooks & lua,
               // don't use the cached bad one next time
               force = true;
               continue;
-            }          
+            }
         }
     }
   if (pkcs8_key)
@@ -345,7 +349,7 @@ change_key_passphrase(lua_hooks & lua,
   shared_ptr<RSA_PrivateKey> priv = get_private_key(lua, id, encoded_key, true);
 
   utf8 new_phrase;
-  get_passphrase(lua, id, new_phrase, true, true, "enter new passphrase");
+  get_passphrase(lua, id, new_phrase, true, true);
 
   Pipe p;
   p.start_msg();
@@ -383,10 +387,13 @@ make_signature(app_state & app,           // to hook for phrase
       || app.opts.ssh_sign == "check"
       || app.opts.ssh_sign == "only")
     {
+      /*
       vector<RSA_PublicKey> ssh_keys = app.agent.get_keys();
       if (ssh_keys.size() <= 0)
         L(FL("make_signature: no rsa keys received from ssh-agent"));
       else {
+      */
+      if (app.agent.connected()) {
         //grab the monotone public key as an RSA_PublicKey
         app.keys.get_key_pair(id, key);
         rsa_pub_key pub;
@@ -401,7 +408,7 @@ make_signature(app_state & app,           // to hook for phrase
 
         if (!pub_key)
           throw informative_failure("Failed to get monotone RSA public key");
-
+        /*
         //if monotone key matches ssh-agent key, sign with ssh-agent
         for (vector<RSA_PublicKey>::const_iterator
                si = ssh_keys.begin(); si != ssh_keys.end(); ++si) {
@@ -409,10 +416,13 @@ make_signature(app_state & app,           // to hook for phrase
               && (*pub_key).get_n() == (*si).get_n()) {
             L(FL("make_signature: ssh key matches monotone key, signing with"
                  " ssh-agent"));
-            app.agent.sign_data(*si, tosign, sig_string);
+        */
+            app.agent.sign_data(*pub_key, tosign, sig_string);
+        /*
             break;
           }
         }
+        */
       }
       if (sig_string.length() <= 0)
         L(FL("make_signature: monotone and ssh-agent keys do not match, will"
