@@ -15,7 +15,7 @@ LUAEXT(match, globish)
     string r(re);
     string n;
     string s(str);
-    result = globish_matcher(r, n)(s);
+    result = globish_matcher(globish(r), globish(n))(s);
   } catch (informative_failure & e) {
     return luaL_error(L, e.what());
   } catch (boost::bad_pattern & e) {
@@ -27,3 +27,13 @@ LUAEXT(match, globish)
   return 1;
 }
 
+
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
+
