@@ -210,48 +210,48 @@ sanity::warning(i18n_format const & i18nfmt,
 }
 
 void
-sanity::naughty_failure(string const & expr, i18n_format const & explain,
-                        string const & file, int line)
+sanity::naughty_failure(char const * expr, i18n_format const & explain,
+                        char const * file, int line)
 {
   string message;
   log(FL("%s:%d: usage constraint '%s' violated") % file % line % expr,
-      file.c_str(), line);
-  prefix_lines_with(_("misuse: "), do_format(explain, file.c_str(), line), message);
+      file, line);
+  prefix_lines_with(_("misuse: "), do_format(explain, file, line), message);
   gasp();
   throw informative_failure(message);
 }
 
 void
-sanity::error_failure(string const & expr, i18n_format const & explain,
-                      string const & file, int line)
+sanity::error_failure(char const * expr, i18n_format const & explain,
+                      char const * file, int line)
 {
   string message;
   log(FL("%s:%d: detected error '%s' violated") % file % line % expr,
-      file.c_str(), line);
-  prefix_lines_with(_("error: "), do_format(explain, file.c_str(), line), message);
+      file, line);
+  prefix_lines_with(_("error: "), do_format(explain, file, line), message);
   throw informative_failure(message);
 }
 
 void
-sanity::invariant_failure(string const & expr,
-                          string const & file, int line)
+sanity::invariant_failure(char const * expr, char const * file, int line)
 {
   char const * pattern = N_("%s:%d: invariant '%s' violated");
-  log(FL(pattern) % file % line % expr, file.c_str(), line);
+  log(FL(pattern) % file % line % expr, file, line);
   gasp();
   throw logic_error((F(pattern) % file % line % expr).str());
 }
 
 void
-sanity::index_failure(string const & vec_expr,
-                      string const & idx_expr,
+sanity::index_failure(char const * vec_expr,
+                      char const * idx_expr,
                       unsigned long sz,
                       unsigned long idx,
-                      string const & file, int line)
+                      char const * file, int line)
 {
-  char const * pattern = N_("%s:%d: index '%s' = %d overflowed vector '%s' with size %d");
+  char const * pattern
+    = N_("%s:%d: index '%s' = %d overflowed vector '%s' with size %d");
   log(FL(pattern) % file % line % idx_expr % idx % vec_expr % sz,
-      file.c_str(), line);
+      file, line);
   gasp();
   throw logic_error((F(pattern)
                      % file % line % idx_expr % idx % vec_expr % sz).str());
