@@ -20,13 +20,13 @@ REV3=base_revision()
 writefile("foo", "foo again\n")
 writefile("bar", "bar again\n")
 
-check(mtn("rename", "-e", "foo", "foo2"), 0, false, false)
+check(mtn("rename", "foo", "foo2"), 0, false, false)
 
 commit()
 REV4=base_revision()
 
 -- without restrictions
-check(mtn("log", "--diffs"), 0, true, false)
+check(mtn("log", "--diffs", "--no-graph"), 0, true, false)
 check(grep('^(---|\\+\\+\\+) ', "stdout"), 0, true, false)
 rename("stdout", "full")
 check(get("expect_full"))
@@ -34,7 +34,7 @@ canonicalize("full")
 check(samefile("expect_full", "full"))
 
 -- restrict to foo2 and quux
-check(mtn("log", "quux", "--diffs", "foo2"), 0, true, false)
+check(mtn("log", "--no-graph", "quux", "--diffs", "foo2"), 0, true, false)
 check(grep("^(---|\\+\\+\\+) ", "stdout"), 0, true, false)
 rename("stdout", "restrict")
 check(get("expect_restrict"))
