@@ -11,7 +11,7 @@ root_r_sha = base_revision()
 root_f_sha = sha1("foo")
 
 -- produce move edge
-check(mtn("rename", "foo", "bar"), 0, false, false)
+check(mtn("rename", "--bookkeep-only", "foo", "bar"), 0, false, false)
 copy("foo", "bar")
 commit()
 
@@ -32,23 +32,23 @@ check(qgrep("bar", "manifest"))
 check(qgrep("bleh", "manifest"))
 
 -- rename a rename target
-check(mtn("rename", "bleh", "blah"), 0, false, false)
+check(mtn("rename", "--bookkeep-only", "bleh", "blah"), 0, false, false)
 check(qgrep("bleh", "_MTN/revision"))
 check(qgrep("blah", "_MTN/revision"))
-check(mtn("rename", "blah", "blyh"), 0, false, false)
+check(mtn("rename", "--bookkeep-only", "blah", "blyh"), 0, false, false)
 check(qgrep("bleh", "_MTN/revision"))
 check(not qgrep("blah", "_MTN/revision"))
 check(qgrep("blyh", "_MTN/revision"))
 
 -- undo a rename
-check(mtn("rename", "blyh", "bleh"), 0, false, false)
+check(mtn("rename", "--bookkeep-only", "blyh", "bleh"), 0, false, false)
 check(not qgrep("blyh", "_MTN/revision"))
 check(not qgrep("bleh", "_MTN/revision"))
 
 -- move file before renaming it
 check(mtn("status"), 0, false, false)
 rename("bar", "barfoo")
-check(mtn("rename", "bar", "barfoo"), 0, false, true)
+check(mtn("rename", "--bookkeep-only", "bar", "barfoo"), 0, false, true)
 check(qgrep('renaming bar to barfoo in workspace manifest', "stderr"))
 check(mtn("status"), 0, false, false)
 
@@ -57,6 +57,6 @@ rename("barfoo", "bar")
 check(mtn("revert", "."), 0, false, false)
 check(mtn("status"), 0, false, false)
 rename("bar", "barfoofoo")
-check(mtn("rename", "bar", "barfoo"), 0, false, true)
+check(mtn("rename", "--bookkeep-only", "bar", "barfoo"), 0, false, true)
 check(qgrep('renaming bar to barfoo in workspace manifest', "stderr"))
 check(mtn("status"), 1, false, false)
