@@ -60,7 +60,10 @@ end
 function merge2(left_path, right_path, merged_path, left, right)
 	io.write("running merge2 hook\n") 
 	return left
-end
+     end
+
+merge3 = nil
+edit_comment = nil
 
 if (attr_functions == nil) then
   attr_functions = {}
@@ -70,66 +73,8 @@ attr_functions["test:test_attr"] =
     io.write(string.format("test:test_attr:%s:%s\n", filename, value))
   end
 
-function get_netsync_connect_command(uri, args)
-
-	local argv = nil
-
-	if uri["scheme"] == "ssh" 
-		and uri["host"] 
-		and uri["path"] then
-
-		argv = { "ssh" }
-		if uri["user"] then
-			table.insert(argv, "-l")
-			table.insert(argv, uri["user"])
-		end
-		if uri["port"] then
-			table.insert(argv, "-p")
-			table.insert(argv, uri["port"])
-		end
-
-		table.insert(argv, uri["host"])
-	end
-	
-	if uri["scheme"] == "file" and uri["path"] then
-		argv = { }
-	end
-
-	if argv then
-
-		table.insert(argv, os.getenv("mtn"))
-
-		if args["debug"] then
-			table.insert(argv, "--debug")
-		else
-			table.insert(argv, "--quiet")
-		end
-
-		table.insert(argv, "--db")
-		table.insert(argv, uri["path"])
-		table.insert(argv, "serve")
-		table.insert(argv, "--stdio")
-		table.insert(argv, "--no-transport-auth")
-
-		if args["include"] then
-			table.insert(argv, args["include"])
-		end
-
-		if args["exclude"] then
-			table.insert(argv, "--exclude")
-			table.insert(argv, args["exclude"])
-		end
-	end
-	return argv
-end
-
-function use_transport_auth(uri)
-	if uri["scheme"] == "ssh" 
-	or uri["scheme"] == "file" then
-		return false
-	else
-		return true
-	end
+function get_mtn_command(host)
+   return os.getenv("mtn")
 end
 
 function get_encloser_pattern(name)
