@@ -18,4 +18,4 @@ check(get("bad.db"))
 
 check(mtn("db", "migrate", "-d", "bad.db"), 0, false, false)
 
-check(mtn("db", "regenerate_rosters", "-d", "bad.db"), 3, false, true)
+check(mtn("db", "regenerate_caches", "-d", "bad.db"), 3, false, true)
