@@ -14,7 +14,7 @@
 #include <sstream>
 #include <string>
 #include <vector>
-#include <wchar.h>
+
 
 #include <boost/tokenizer.hpp>
 #include <boost/scoped_array.hpp>
@@ -60,99 +60,80 @@ using boost::scoped_array;
 // paradigm "must" be used. this program is intended for source code
 // control and I make no bones about it.
 
-// the generic function
-template<typename XFM> string xform(string const & in)
+NORETURN(static inline void error_in_transform(Botan::Exception & e));
+
+static inline void
+error_in_transform(Botan::Exception & e)
+{
+  // why do people make up their own out-of-memory exceptions?
+  if (typeid(e) == typeid(Botan::Memory_Exhaustion))
+    throw std::bad_alloc();
+
+  // these classes can all indicate data corruption
+  else if (typeid(e) == typeid(Botan::Encoding_Error)
+           || typeid(e) == typeid(Botan::Decoding_Error)
+           || typeid(e) == typeid(Botan::Stream_IO_Error)
+           || typeid(e) == typeid(Botan::Integrity_Failure))
+    {
+      // clean up the what() string a little: throw away the
+      // "botan: TYPE: " part...
+      string w(e.what());
+      string::size_type pos = w.find(':');
+      pos = w.find(':', pos+1);
+      w = string(w.begin() + pos + 2, w.end());
+
+      // ... downcase the rest of it and replace underscores with spaces.
+      for (string::iterator p = w.begin(); p != w.end(); p++)
+        if (::isupper(*p))
+          *p = ::tolower(*p);
+        else if (*p == '_')
+          *p = ' ';
+
+      E(false,
+        F("%s\n"
+          "this may be due to a memory glitch, data corruption during\n"
+          "a network transfer, corruption of your database or workspace,\n"
+          "or a bug in monotone.  if the error persists, please contact\n"
+          "%s for assistance.\n")
+        % w % PACKAGE_BUGREPORT);
+    }
+  else
+    throw;
+
+  I(false);  // can't get here
+}
+
+// worker function for the visible functions below
+namespace {
+template<typename XFM> string xform(XFM * x, string const & in)
 {
   string out;
-  Botan::Pipe pipe(new XFM());
-  pipe.process_msg(in);
-  out = pipe.read_all_as_string();
+  try
+    {
+      Botan::Pipe pipe(x);
+      pipe.process_msg(in);
+      out = pipe.read_all_as_string();
+    }
+  catch (Botan::Exception & e)
+    {
+      error_in_transform(e);
+    }
   return out;
 }
-
-// specialize it
-template string xform<Botan::Base64_Encoder>(string const &);
-template string xform<Botan::Base64_Decoder>(string const &);
-template string xform<Botan::Hex_Encoder>(string const &);
-template string xform<Botan::Hex_Decoder>(string const &);
-template string xform<Botan::Gzip_Compression>(string const &);
-template string xform<Botan::Gzip_Decompression>(string const &);
-
-// for use in hexenc encoding
-
-static inline void
-encode_hexenc_inner(string::const_iterator i,
-                    string::const_iterator end,
-                    char *out)
-{
-  static char const *tab = "0123456789abcdef";
-  for (; i != end; ++i)
-    {
-      *out++ = tab[(*i >> 4) & 0xf];
-      *out++ = tab[*i & 0xf];
-    }
 }
 
+// full specializations for the usable cases of xform<XFM>()
+// use extra error checking in base64 and hex decoding
+#define SPECIALIZE_XFORM(T, carg) \
+  template<> string xform<T>(string const &in) \
+  { return xform(new T(carg), in); }
 
-string encode_hexenc(string const & in)
-{
-  if (LIKELY(in.size() == constants::idlen / 2))
-    {
-      char buf[constants::idlen];
-      encode_hexenc_inner(in.begin(), in.end(), buf);
-      return string(buf, constants::idlen);
-    }
-  else
-    {
-      scoped_array<char> buf(new char[in.size() * 2]);
-      encode_hexenc_inner(in.begin(), in.end(), buf.get());
-      return string(buf.get(), in.size() *2);
-    }
-}
-
-static inline char
-decode_hex_char(char c)
-{
-  if (c >= '0' && c <= '9')
-    return c - '0';
-  else
-  {
-          I(c >= 'a' && c <= 'f');
-          return c - 'a' + 10;
-  }
-}
-
-static inline void
-decode_hexenc_inner(string::const_iterator i,
-                    string::const_iterator end,
-                    char *out)
-{
-  for (; i != end; ++i)
-    {
-      char t = decode_hex_char(*i++);
-      t <<= 4;
-      t |= decode_hex_char(*i);
-      *out++ = t;
-    }
-}
-
-string decode_hexenc(string const & in)
-{
-
-  I(in.size() % 2 == 0);
-  if (LIKELY(in.size() == constants::idlen))
-    {
-      char buf[constants::idlen / 2];
-      decode_hexenc_inner(in.begin(), in.end(), buf);
-      return string(buf, constants::idlen / 2);
-    }
-  else
-    {
-      scoped_array<char> buf(new char[in.size() / 2]);
-      decode_hexenc_inner(in.begin(), in.end(), buf.get());
-      return string(buf.get(), in.size() / 2);
-    }
-}
+SPECIALIZE_XFORM(Botan::Base64_Encoder,);
+SPECIALIZE_XFORM(Botan::Base64_Decoder, Botan::IGNORE_WS);
+SPECIALIZE_XFORM(Botan::Hex_Encoder, Botan::Hex_Encoder::Lowercase);
+SPECIALIZE_XFORM(Botan::Hex_Decoder, Botan::IGNORE_WS);
+SPECIALIZE_XFORM(Botan::Gzip_Compression,);
+SPECIALIZE_XFORM(Botan::Gzip_Decompression,);
 
 template <typename T>
 void pack(T const & in, base64< gzip<T> > & out)
@@ -160,10 +141,18 @@ void pack(T const & in, base64< gzip<T> > & out)
   string tmp;
   tmp.reserve(in().size()); // FIXME: do some benchmarking and make this a constant::
 
-  Botan::Pipe pipe(new Botan::Gzip_Compression(), new Botan::Base64_Encoder);
-  pipe.process_msg(in());
-  tmp = pipe.read_all_as_string();
-  out = tmp;
+  try
+    {
+      Botan::Pipe pipe(new Botan::Gzip_Compression(),
+                       new Botan::Base64_Encoder);
+      pipe.process_msg(in());
+      tmp = pipe.read_all_as_string();
+      out = base64< gzip<T> >(tmp);
+    }
+  catch (Botan::Exception & e)
+    {
+      error_in_transform(e);
+    }
 }
 
 template <typename T>
@@ -172,11 +161,18 @@ void unpack(base64< gzip<T> > const & in, T & out)
   string tmp;
   tmp.reserve(in().size()); // FIXME: do some benchmarking and make this a constant::
 
-  Botan::Pipe pipe(new Botan::Base64_Decoder(), new Botan::Gzip_Decompression());
-  pipe.process_msg(in());
-  tmp = pipe.read_all_as_string();
-
-  out = tmp;
+  try
+    {
+      Botan::Pipe pipe(new Botan::Base64_Decoder(),
+                       new Botan::Gzip_Decompression());
+      pipe.process_msg(in());
+      tmp = pipe.read_all_as_string();
+      out = T(tmp);
+    }
+  catch (Botan::Exception & e)
+    {
+      error_in_transform(e);
+    }
 }
 
 // specialise them
@@ -204,7 +200,7 @@ patch(data const & olddata,
 {
   string result;
   apply_delta(olddata(), del(), result);
-  newdata = result;
+  newdata = data(result);
 }
 
 // identifier (a.k.a. sha1 signature) calculation
@@ -213,22 +209,17 @@ void
 calculate_ident(data const & dat,
                 hexenc<id> & ident)
 {
-  Botan::Pipe p(new Botan::Hash_Filter("SHA-160"));
-  p.process_msg(dat());
-
-  id ident_decoded(p.read_all_as_string());
-  encode_hexenc(ident_decoded, ident);
-}
-
-void
-calculate_ident(base64< gzip<data> > const & dat,
-                hexenc<id> & ident)
-{
-  gzip<data> data_decoded;
-  data data_decompressed;
-  decode_base64(dat, data_decoded);
-  decode_gzip(data_decoded, data_decompressed);
-  calculate_ident(data_decompressed, ident);
+  try
+    {
+      Botan::Pipe p(new Botan::Hash_Filter("SHA-160"),
+                    new Botan::Hex_Encoder(Botan::Hex_Encoder::Lowercase));
+      p.process_msg(dat());
+      ident = hexenc<id>(p.read_all_as_string());
+    }
+  catch (Botan::Exception & e)
+    {
+      error_in_transform(e);
+    }
 }
 
 void
@@ -237,7 +228,7 @@ calculate_ident(file_data const & dat,
 {
   hexenc<id> tmp;
   calculate_ident(dat.inner(), tmp);
-  ident = tmp;
+  ident = file_id(tmp);
 }
 
 void
@@ -246,7 +237,7 @@ calculate_ident(manifest_data const & dat,
 {
   hexenc<id> tmp;
   calculate_ident(dat.inner(), tmp);
-  ident = tmp;
+  ident = manifest_id(tmp);
 }
 
 void
@@ -255,14 +246,23 @@ calculate_ident(revision_data const & dat,
 {
   hexenc<id> tmp;
   calculate_ident(dat.inner(), tmp);
-  ident = tmp;
+  ident = revision_id(tmp);
 }
 
 string
 canonical_base64(string const & s)
 {
-  return xform<Botan::Base64_Encoder>
-    (xform<Botan::Base64_Decoder>(s));
+  try
+    {
+      Botan::Pipe pipe(new Botan::Base64_Decoder(),
+                       new Botan::Base64_Encoder());
+      pipe.process_msg(s);
+      return pipe.read_all_as_string();
+    }
+  catch (Botan::Exception & e)
+    {
+      error_in_transform(e);
+    }
 }
 
 
@@ -304,6 +304,24 @@ UNIT_TEST(transform, calculate_ident)
   BOOST_CHECK(output() == ident);
 }
 
+UNIT_TEST(transform, corruption_check)
+{
+  data input(string("i'm so fragile, fragile when you're here"));
+  gzip<data> gzd;
+  encode_gzip(input, gzd);
+
+  // fake a single-bit error
+  string gzs = gzd();
+  string::iterator i = gzs.begin();
+  while (*i != '+')
+    i++;
+  *i = 'k';
+
+  gzip<data> gzbad(gzs);
+  data output;
+  BOOST_CHECK_THROW(decode_gzip(gzbad, output), informative_failure);
+}
+
 #endif // BUILD_UNIT_TESTS
 
 // Local Variables:
