@@ -37,6 +37,14 @@ read_password(std::string const & prompt, char * buf, size_t bufsz)
   std::cout.flush();
   echo_off(save_term);
   std::cin.getline(buf, bufsz, '\n');
+  std::cout << std::endl;
   echo_on(save_term);
 }
 
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
