@@ -83,7 +83,7 @@ ostream & operator<<(ostream & ost, insn const & i)
 
 static inline void
 init_match_table(string const & a,
-                 string::size_type blocksz,
+                 const string::size_type blocksz,
                  match_table & tab)
 {
   string::size_type sz = a.size();
@@ -134,9 +134,9 @@ find_match(match_table const & matches,
   string::const_iterator bi = b.begin() + bpos + tlen;
   string::const_iterator be = b.end();
 
-  while((ai != ae)
-        && (bi != be)
-        && (*ai == *bi))
+  while ((ai != ae)
+         && (bi != be)
+         && (*ai == *bi))
     {
       ++tlen;
       ++ai;
@@ -149,10 +149,10 @@ find_match(match_table const & matches,
   // see if we can extend backwards into a previous insert hunk
   if (! delta.empty() && delta.back().code == insn::insert)
     {
-      while(apos > 0
-            && bpos > 0
-            && a[apos - 1] == b[bpos - 1]
-            && !delta.back().payload.empty())
+      while (apos > 0
+             && bpos > 0
+             && a[apos - 1] == b[bpos - 1]
+             && !delta.back().payload.empty())
         {
           I(a[apos - 1] == *(delta.back().payload.rbegin()));
           I(delta.back().payload.size() > 0);
@@ -182,7 +182,19 @@ insert_insn(vector<insn> & delta, char c)
   if (delta.empty() || delta.back().code == insn::copy)
     delta.push_back(insn(c));
   else
-    delta.back().payload += c;
+    {
+      // design in gcc 3.3 and 4.0 STL expands the string one character
+      // at a time when appending single characters ?!
+      // see libstdc++5-3.3-dev 3.3.5-13: basic_string.h:471, calling with
+      // size_type(1), then basic_string.tcc:717 reserving one more byte
+      // if needed.
+      // see libstdc++6-4.0-dev 4.0.3-3: basic_string.h:770 calling push_back
+      // then basic_string.h: 849 again adding 1 to the length.
+      if (delta.back().payload.capacity() == delta.back().payload.size())
+        // standard amortized constant rule
+        delta.back().payload.reserve(delta.back().payload.size() * 2);
+      delta.back().payload += c;
+    }
 }
 
 
@@ -198,7 +210,7 @@ compute_delta_insns(string const & a,
                     string const & b,
                     vector<insn> & delta)
 {
-  string::size_type blocksz = 64;
+  static const string::size_type blocksz = 64;
   match_table matches;
   init_match_table(a, blocksz, matches);
 
@@ -213,17 +225,57 @@ compute_delta_insns(string const & a,
 
   for (string::size_type
        sz = b.size(),
-       lo = 0,
-       hi = blocksz;
+       lo = 0; 
        lo < sz; )
     {
       string::size_type apos = 0, alen = 1, badvance = 1;
 
       bool found_match = find_match(matches, delta, rolling, a, b, lo, apos, alen, badvance);
 
+      // There are basically three cases:
+      // 1) advance by 1 (common, no match found)
+      // 2) advance by >blocksz (semi-common, usual case when a match is found)
+      // 3) advance by <blocksz (rare, unusual case when a match is found)
+      // in case (2), all of the rolling checksum data will be entirely replaced, so
+      // we can do a fast skip forward.
       if (found_match)
         {
           copy_insn(delta, apos, alen);
+          u32 save_lo = lo;
+          if (badvance <= blocksz) 
+            {
+              string::size_type next = lo;
+              I(next < b.size() && (lo + badvance - 1) < b.size());
+              for (; next < lo + badvance; ++next)
+                {
+                  rolling.out(static_cast<u8>(b[next]));
+                  if (next + blocksz < b.size())
+                    rolling.in(static_cast<u8>(b[next + blocksz]));
+                }
+              lo = next;
+            }
+
+          // Skip advancement is always correct; however, for a small
+          // increment it is more expensive than incremental advancement
+          // Cost of doing a in() + out() is roughly the same as doing a
+          // replace_with for 1 character, so if we are advancing more than
+          // blocksz/2, it will be better to do a replacement than an
+          // incremental advance.  The out could be more expensive because
+          // it does a multiply, but for now, ignore this; it turns out that
+          // advancements in the range of [2..blocksz-1] are actually really
+          // rare.
+          if (badvance >= blocksz/2) 
+            {
+              u32 new_lo = save_lo + badvance;
+              u32 new_hi = new_lo + blocksz;
+              if (new_hi > b.size()) 
+                {
+                  new_hi = b.size();
+                }
+              I(new_lo <= new_hi);
+              rolling.replace_with(reinterpret_cast<u8 const *>(b.data() + new_lo), new_hi-new_lo);
+              lo = new_lo;
+            }
         }
       else
         {
@@ -232,24 +284,19 @@ compute_delta_insns(string const & a,
           I(alen < blocksz);
           I(lo < b.size());
           insert_insn(delta, b[lo]);
+          rolling.out(static_cast<u8>(b[lo]));
+          if (lo + blocksz < b.size()) 
+            {
+              rolling.in(static_cast<u8>(b[lo+blocksz]));
+            }
+          ++lo;
         }
-
-      string::size_type next = lo;
-      for (; next < lo + badvance; ++next)
-        {
-          I(next < b.size());
-          rolling.out(static_cast<u8>(b[next]));
-          if (next + blocksz < b.size())
-            rolling.in(static_cast<u8>(b[next + blocksz]));
-        }
-      lo = next;
-      hi = lo + blocksz;
     }
 }
 
 void
 write_delta_insns(vector<insn> const & delta_insns,
-		  string & delta)
+                  string & delta)
 {
   delta.clear();
   ostringstream oss;
@@ -274,16 +321,12 @@ compute_delta(string const & a,
   // xdelta. several places of the xdelta code prefer assertions which are
   // only true with non-empty chunks anyways.
 
-  if (a == b)
-    {
-      delta.clear();
-      return;
-    }
-
   if (a.size() == 0 && b.size() != 0)
     delta_insns.push_back(insn(b));
   else if (a.size() != 0 && b.size() == 0)
     delta_insns.push_back(insn(0, 0));
+  else if (a == b)
+    delta_insns.push_back(insn(0, a.size()));
   else
     {
       I(a.size() > 0);
@@ -369,13 +412,13 @@ apply_delta(shared_ptr<delta_applicator> da,
           I(*i == '\n');
           ++i;
           I(i != delta.end());
-	  I((i - delta.begin()) + len <= delta.size());
-	  if (len > 0)
-	    {
-	      string tmp(i, i+len);
-	      da->insert(tmp);
-	    }
-	  i += len;
+          I((i - delta.begin()) + len <= delta.size());
+          if (len > 0)
+            {
+              string tmp(i, i+len);
+              da->insert(tmp);
+            }
+          i += len;
         }
       else
         {
@@ -385,8 +428,8 @@ apply_delta(shared_ptr<delta_applicator> da,
           string::size_type pos = read_num(i, delta.end());
           I(i != delta.end());
           string::size_type len = read_num(i, delta.end());
-	  if (len != 0)
-	    da->copy(pos, len);
+          if (len != 0)
+            da->copy(pos, len);
         }
       I(i != delta.end());
       I(*i == '\n');
@@ -407,35 +450,6 @@ apply_delta(string const & a,
   da->finish(b);
 }
 
-struct
-size_accumulating_delta_applicator :
-  public delta_applicator
-{
-  u64 & sz;
-  size_accumulating_delta_applicator(u64 & s) : sz(s) {}
-  virtual void begin(string const & base) {}
-  virtual void next() {}
-  virtual void finish(string & out) {}
-
-  virtual void copy(string::size_type pos,
-                    string::size_type len)
-  { sz += len; }
-  virtual void insert(string const & str)
-  { sz += str.size(); }
-};
-
-
-u64
-measure_delta_target_size(string const & delta)
-{
-  u64 sz = 0;
-  shared_ptr<delta_applicator> da(new size_accumulating_delta_applicator(sz));
-  apply_delta(da, delta);
-  return sz;
-}
-
-
-
 // piecewise-applicator stuff follows (warning: ugly)
 
 typedef string::size_type version_pos;
@@ -496,11 +510,12 @@ piece_table
   void build(version_spec const & in, string & out)
   {
     out.clear();
-    for (version_spec::const_iterator i = in.begin();
-         i != in.end(); ++i)
-      {
-        append(out, i->piece, i->ppos, i->len);
-      }
+    unsigned out_len = 0;
+    for (version_spec::const_iterator i = in.begin(); i != in.end(); ++i)
+      out_len += i->len;
+    out.reserve(out_len);
+    for (version_spec::const_iterator i = in.begin(); i != in.end(); ++i)
+      append(out, i->piece, i->ppos, i->len);
   }
 };
 
@@ -673,8 +688,8 @@ new_piecewise_applicator()
 struct copied_extent
 {
   copied_extent(string::size_type op,
-		string::size_type np,
-		string::size_type len)
+                string::size_type np,
+                string::size_type len)
     : old_pos(op),
       new_pos(np),
       len(len)
@@ -720,32 +735,32 @@ inverse_delta_writing_applicator :
     vector<insn> delta_insns;
 
     for (set<copied_extent>::iterator i = copied_extents.begin();
-	 i != copied_extents.end(); ++i)
-      {	
-	// It is possible that this extent left a gap after the
-	// previously copied extent; in this case we wish to pad
-	// the intermediate space with an insert.
-	while (old_pos < i->old_pos)
-	  {
-	    I(old_pos < old.size());
-	    // Don't worry, adjacent inserts are merged.
-	    insert_insn(delta_insns, old.at(old_pos++));
-	  }
+         i != copied_extents.end(); ++i)
+      { 
+        // It is possible that this extent left a gap after the
+        // previously copied extent; in this case we wish to pad
+        // the intermediate space with an insert.
+        while (old_pos < i->old_pos)
+          {
+            I(old_pos < old.size());
+            // Don't worry, adjacent inserts are merged.
+            insert_insn(delta_insns, old.at(old_pos++));
+          }
 
-	// It is also possible that this extent *overlapped* the
-	// previously copied extent; in this case we wish to subtract
-	// the overlap from the inverse copy.
+        // It is also possible that this extent *overlapped* the
+        // previously copied extent; in this case we wish to subtract
+        // the overlap from the inverse copy.
 
-	string::size_type overlap = 0;
-	if (i->old_pos < old_pos)
-	  overlap = old_pos - i->old_pos;
+        string::size_type overlap = 0;
+        if (i->old_pos < old_pos)
+          overlap = old_pos - i->old_pos;
 
-	if (i->len <= overlap)
-	  continue;
+        if (i->len <= overlap)
+          continue;
 
-	I(i->len > overlap);
-	copy_insn(delta_insns, i->new_pos + overlap, i->len - overlap);
-	old_pos += (i->len - overlap);
+        I(i->len > overlap);
+        copy_insn(delta_insns, i->new_pos + overlap, i->len - overlap);
+        old_pos += (i->len - overlap);
       }
     while (old_pos < old.size())
       insert_insn(delta_insns, old.at(old_pos++));
@@ -770,8 +785,8 @@ inverse_delta_writing_applicator :
 
 void
 invert_xdelta(string const & old_str,
-	      string const & delta,
-	      string & delta_inverse)
+              string const & delta,
+              string & delta_inverse)
 {
   shared_ptr<delta_applicator> da(new inverse_delta_writing_applicator(old_str));
   apply_delta(da, delta);
@@ -782,6 +797,58 @@ invert_xdelta(string const & old_str,
 #ifdef BUILD_UNIT_TESTS
 
 #include "unit_tests.hh"
+
+string
+apply_via_normal(string const & base, string const & delta)
+{
+  string tmp;
+  apply_delta(base, delta, tmp);
+  return tmp;
+}
+
+string
+apply_via_piecewise(string const & base, string const & delta)
+{
+  shared_ptr<delta_applicator> appl = new_piecewise_applicator();
+  appl->begin(base);
+  apply_delta(appl, delta);
+  appl->next();
+  string tmp;
+  appl->finish(tmp);
+  return tmp;
+}
+
+void
+spin(string a, string b)
+{
+  string ab, ba;
+  compute_delta(a, b, ab);
+  compute_delta(b, a, ba);
+  BOOST_CHECK(a == apply_via_normal(b, ba));
+  BOOST_CHECK(a == apply_via_piecewise(b, ba));
+  BOOST_CHECK(b == apply_via_normal(a, ab));
+  BOOST_CHECK(b == apply_via_piecewise(a, ab));
+  string ab_inverted, ba_inverted;
+  invert_xdelta(a, ab, ab_inverted);
+  invert_xdelta(b, ba, ba_inverted);
+  BOOST_CHECK(a == apply_via_normal(b, ab_inverted));
+  BOOST_CHECK(a == apply_via_piecewise(b, ab_inverted));
+  BOOST_CHECK(b == apply_via_normal(a, ba_inverted));
+  BOOST_CHECK(b == apply_via_piecewise(a, ba_inverted));
+}
+
+UNIT_TEST(xdelta, simple_cases)
+{
+  L(FL("empty/empty"));
+  spin("", "");
+  L(FL("empty/short"));
+  spin("", "a");
+  L(FL("empty/longer"));
+  spin("", "asdfasdf");
+  L(FL("two identical strings"));
+  spin("same string", "same string");
+}
+
 #ifdef WIN32
 #define BOOST_NO_STDC_NAMESPACE
 #endif
@@ -809,7 +876,7 @@ xdelta_random_string(string & str)
   size_t sz = xdelta_sizegen(PRNG);
   str.clear();
   str.reserve(sz);
-  while(sz-- > 0)
+  while (sz-- > 0)
     {
       str += xdelta_chargen(PRNG);
     }
@@ -865,45 +932,82 @@ xdelta_randomly_delete(string & str)
     }
 }
 
-void
-xdelta_random_simple_delta_test()
+UNIT_TEST(xdelta, random_simple_delta)
 {
   for (int i = 0; i < 100; ++i)
     {
-      string a, b, fdel, rdel, c, d;
+      string a, b;
       xdelta_random_string(a);
       b = a;
       xdelta_randomly_change(b);
       xdelta_randomly_insert(b);
       xdelta_randomly_delete(b);
-      compute_delta(a, b, fdel);
-      compute_delta(b, a, rdel);
-      L(FL("src %d, dst %d, fdel %d, rdel %d")
-        % a.size() % b.size()% fdel.size() % rdel.size()) ;
-      if (fdel.size() == 0)
-        {
-          L(FL("confirming src == dst and rdel == 0"));
-          BOOST_CHECK(a == b);
-          BOOST_CHECK(rdel.size() == 0);
-        }
-      else
-        {
-          apply_delta(a, fdel, c);
-          apply_delta(b, rdel, d);
-          L(FL("confirming dst1 %d, dst2 %d") % c.size() % d.size());
-          BOOST_CHECK(b == c);
-          BOOST_CHECK(a == d);
-        }
+      spin(a, b);
     }
 }
 
-void
-add_xdelta_tests(test_suite * suite)
+UNIT_TEST(xdelta, random_piecewise_delta)
 {
-  I(suite);
-  suite->add(BOOST_TEST_CASE(&xdelta_random_simple_delta_test));
+  for (int i = 0; i < 50; ++i)
+    {
+      string prev, next, got;
+      xdelta_random_string(prev);
+      shared_ptr<delta_applicator> appl = new_piecewise_applicator();
+      appl->begin(prev);
+      for (int j = 0; j < 5; ++j)
+        {
+          appl->finish(got);
+          BOOST_CHECK(got == prev);
+          next = prev;
+          xdelta_randomly_change(next);
+          xdelta_randomly_insert(next);
+          xdelta_randomly_delete(next);
+          string delta;
+          compute_delta(prev, next, delta);
+          apply_delta(appl, delta);
+          appl->next();
+          prev = next;
+        }
+      appl->finish(got);
+      BOOST_CHECK(got == prev);
+  }
 }
 
+UNIT_TEST(xdelta, rolling_sanity_check)
+{
+  const unsigned testbufsize = 512;
+  static const string::size_type blocksz = 64;
+  char testbuf[testbufsize];
+
+  for(unsigned i = 0; i < testbufsize; ++i) 
+    {
+      testbuf[i] = xdelta_chargen(PRNG);
+    }
+  for(unsigned advanceby = 0; advanceby < testbufsize; ++advanceby) 
+    {
+      adler32 incremental(reinterpret_cast<u8 const *>(testbuf), blocksz);
+      for(unsigned i = 0; i < advanceby; ++i) 
+        {
+          incremental.out(static_cast<u8>(testbuf[i]));
+          if ((i + blocksz) < testbufsize) 
+            {
+              incremental.in(static_cast<u8>(testbuf[i+blocksz]));
+            }
+        }
+      adler32 skip(reinterpret_cast<u8 const *>(testbuf), blocksz);
+      u32 new_lo = advanceby;
+      u32 new_hi = new_lo + blocksz;
+      if (new_hi > testbufsize) 
+        {
+          new_hi = testbufsize;
+        }
+      skip.replace_with(reinterpret_cast<u8 const *>(testbuf + new_lo), new_hi - new_lo);
+
+      BOOST_CHECK(skip.sum() == incremental.sum());
+    }
+  L(FL("rolling sanity check passed"));
+}                   
+
 #endif // BUILD_UNIT_TESTS
 
 // Local Variables:
